﻿local MAX_TRIES = 10 -- The number of times the AddOn will try to select a random track different to the previous one before using the last selection anyway
local DELAY = 0 -- The number of seconds to wait between a track ending and a new one being played.

-- Known Issue: (Beta Build 16030 and Live Build 16016 - 2012-09-02) If DELAY is greater than 0, the current track will start to repeat itself before the next one is played.
-- (DELAY = 0 means the next track is played before this can happen)
-- This happens even when the Sound_ZoneMusicNoDelay CVar is set to 0 (i.e. the "Loop Music" option disabled).

-------------------
-- END OF CONFIG --
-------------------

local random = math.random

local addon, ns = ...
local MUSIC = ns.MUSIC

local NUM_TRACKS = #MUSIC > 0 and #MUSIC or 1
local MUSIC_LOOP_CVAR = "Sound_ZoneMusicNoDelay"

IS_PLAYING = false
local CURRENT_TIMER;
local PREVIOUS_TRACK_INDEX;

local PBM = CreateFrame("Frame")
PBM:SetScript("OnEvent", function(self, event, ...)
	print(debugprofilestop(), "Event:", event)
	self[event](self, ...)
end)

LibStub("AceTimer-3.0"):Embed(PBM)

-----------
-- Music --
-----------

PBM:RegisterEvent("PET_BATTLE_OPENING_START") -- Fired when the pet battle starts
PBM:RegisterEvent("PET_BATTLE_OVER") -- Fired at the end of the pet battle (before PET_BATTLE_CLOSE, which seems to fire multiple times)

local PREV_TIME = debugprofilestop()
local MUSIC_LOOP_VALUE;

function PBM:PlayNextTrack()
	local t = debugprofilestop()
	local diff = t - PREV_TIME
	print(debugprofilestop(), "PBM: TimeDiff", diff)
	local trackIndex, trackInfo;
	local tries = 0
	repeat
		tries = tries + 1
		trackIndex = random(NUM_TRACKS)
		trackInfo = MUSIC[trackIndex]
	until (trackInfo and trackIndex ~= PREVIOUS_TRACK_INDEX) or tries > MAX_TRIES -- Select a random track, but make sure it's not the current one (only try this MAX_TRIES times then break)
	
	if trackInfo then
		PlayMusic(trackInfo[1]) -- trackInfo[1] is the path to the sound file
		print(debugprofilestop(), "PBM: PlayMusic", trackInfo[1], trackInfo[2])
		-- trackInfo[2] is the length of the file in seconds, we schedule the next timer to go off DELAY seconds after the current track ends
		CURRENT_TIMER = self:ScheduleTimer("PlayNextTrack", trackInfo[2] + DELAY)
		PREVIOUS_TRACK_INDEX = trackIndex
		IS_PLAYING = true
	else
		CURRENT_TIMER = nil -- We couldn't find a track to play (probably because ns.MUSIC is empty)
		-- PREVIOUS_TRACK_INDEX 
		IS_PLAYING = false
	end
	PREV_TIME = t
end

function PBM:StopMusic()
	if IS_PLAYING then
		StopMusic()
	end
	if CURRENT_TIMER then
		self:CancelTimer(CURRENT_TIMER)
		CURRENT_TIMER = nil
	end
	IS_PLAYING = false
end

function PBM:PET_BATTLE_OPENING_START()
	MUSIC_LOOP_VALUE = GetCVar(MUSIC_LOOP_CVAR)
	SetCVar(MUSIC_LOOP_CVAR, "0")
	self:PlayNextTrack()
end

function PBM:PET_BATTLE_OVER()
	self:StopMusic()
	SetCVar(MUSIC_LOOP_CVAR, MUSIC_LOOP_VALUE)
end

-------------------------
-- Announcements (WIP) --
-------------------------

--[[ -- This part isn't complete yet.

PBM:RegisterEvent("PET_BATTLE_HEALTH_CHANGED") -- Fired when a pet's health changes (due to damage or healing)

local PET_FACTIONS = {
	[LE_BATTLE_PET_WEATHER] = "Weather",
	[LE_BATTLE_PET_ALLY] = "Ally",
	[LE_BATTLE_PET_ENEMY] = "Enemy",
}
	
	

function PBM:PET_BATTLE_HEALTH_CHANGED(dest, source, amount)
	local eventType;
	if amount > 0 then
		eventType = "heal"
	else
		eventType = "damage"
	end
	
	local sourceStr = PET_FACTIONS[source]
	local destStr = PET_FACTIONS[dest]
	local actionStr = amount > 0 and "Damaged" or "Healed"
	
	local methodName = sourceStr .. actionStr .. destStr
	self[methodName](self, amount)
end

--]]